"""
    
    Complex SummaryField usage example
    
    We have three levels of object
        - Patient
            - Treatment
                - Visit
                
    Patient has a summary field which generates summary from 
    all treatment and visits.
    
    This is provided for testing and example.
    
    Copyright 2007 Red Innovation http://www.redinnovation.com
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Zope imports
import Acquisition
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass

# Plone imports
from Products.Archetypes.interfaces import IBaseFolder
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Registry import registerField

from Products.Archetypes.public import *
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import ATCTOrderedFolder, ATCTFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.summary.core import *
from Products.summary.summarizers import *
from config import *

def getVisitsRecursively(patient):
    """ Helper method to get the all visits of a patient """
    visits = []
    for treatment in patient.listFolderContents():
        for visit in treatment.listFolderContents():
            visits.append(visit)
            
    return visits

class TotalCostSummarizer(DummySummarizer):
    """ Create a column which calculates total cost of all visit costs """
            
    def __init__(self, patient):
        self.patient = patient
        
    def fetchCellValue(self, data, col, row, object, fieldId, mode):  
        """ Return cell value in get() """        
        visits = getVisitsRecursively(self.patient)
        total = 0
        for visit in visits:
            # Data might not be initialized yet
            if visit.getCost() != None:
                total += visit.getCost()
        
        # Update table value
        data.setValue(col, row, total)

class ComplexSummaryField(LockFilledSummaryField):
    """ Display patient summary data
    
    This is a demo type showing which methods you need override
    to create a custom data backend for SummaryField.
    
    Each column is a Visit object. There can be n columns.
    
    Each row is predefined field whose value comes either from
    Treatment object or Visit object.
    
    """
    
    # Fields which are picked up from SummaryDemoTreatment    
    # to row headers. (id, Friendly Name) tuples
    treatmentFields = [ ("diseaseName", "Disase name") ]
    
    # Fields which are picked up from SummaryDemoVisit to
    # row headers
    visitFields = [
       ("bloodPressure", "Blood pressure"), 
       ("cost", "Visit cost"),
       ("patientAction", "Action")
        ]
    
    # All rows in this summary field
    rows = treatmentFields + visitFields        
    
    def initializeSummaryData(self, instance, mode):
        """ Prepares SummaryData instance
        
        Builds table structure        
            - Column labels
            - Row labels
            - Cell data types
        """
        
        data = SummaryData()
                    
        # Add a new column for each visit
        # Each column label will be the creation date of the visit object
        # (visit date)
        visits = getVisitsRecursively(instance)
        for visit in visits:
            data.addColumn(visit.getId(), str(visit.created()))

        # Calculated total cost column appears only in view mode    
        if mode == SummaryField.SUMMARY_VIEW:
            # Add a total cost column
            #print "Adding column total"            
            data.addColumn("total", "Total")             
                                    
        # Initialize row labels
        # from field definitions
        for row in self.rows:     
            data.addRow(row[0], row[1])
        
        # Initialize cell types
        visits = getVisitsRecursively(instance)
        for visit in visits:                        
            for row in self.rows:
                fieldId = row[0] # id, name tuples
                if self.isTreatmentField(fieldId):
                    # Use treatment object as the source for fields
                    object = Acquisition.aq_parent(visit)
                else:
                    object = visit
                    
                # col = visit id
                # row = field id
                self.initCell(data, visit.getId(), fieldId, object, fieldId)                
        
        # Calculated total cost column appears only in view mode    
        if mode == SummaryField.SUMMARY_VIEW:               
            # Initialize total cost column
            for row in self.rows:
                fieldId = row[0]           
                if fieldId == "cost":
                    data.setSummarizer("total", "cost", TotalCostSummarizer(instance))
                else:
                    data.setSummarizer("total", fieldId, DummySummarizer())
                                
        return data
    
    def isTreatmentField(self, fieldId):
        """ Based on row id, field belongs to treatment or visit """
        for f in self.treatmentFields:
            if f[0] == fieldId:
                return True
        return False
                
    def get(self, instance, **kwargs):
        """ Fill in view/form default data """
        
        if not IBaseFolder.isImplementedBy(instance):
            raise ValueError, "Summary field can be placed on folderish objects only"
                        
        # Templates use special mode to generate edit table structure
        if "mode" in kwargs:
            mode = kwargs["mode"]
        else:
            mode=SummaryField.SUMMARY_VIEW        
        
        #print "Got mode " + mode
        data = self.initializeSummaryData(instance, mode)
                        
        # Initialize cell types
        visits = getVisitsRecursively(instance)
        for visit in visits:     
            col = visit.getId()                   
            for row in self.rows:
                
                fieldId = row[0]
                
                if self.isTreatmentField(fieldId):
                    # Use treatment object as the source for fields
                    object = Acquisition.aq_parent(visit)
                else:
                    object = visit
                    
                self.fetchCellValue(data, col, fieldId, object, fieldId, mode)
                
                # We are retrieving info for the edit table structure
                if mode == SummaryField.SUMMARY_EDIT:
                    # Lock already filled values
                    if data.getValue(col, fieldId) == "" or data.getValue(col, fieldId) == None:
                        # This value is empty and the user needs fill it in
                        pass
                    else:
                        # We need to lock the cell to prevent accidental edits
                        data.setLocked(col, fieldId, True)
                        
        if mode == SummaryField.SUMMARY_VIEW:
            # Render Total column        
            for row in self.rows:
                fieldId = row[0]                
                self.fetchCellValue(data, "total", fieldId, None, None, mode)
                                                                                        
        return data

    def set(self, instance, value, **kwargs):
        """ Store form data in host objects """                          
        
        if value == None:
            # No summarized objects yet
            return         
                
        assert isinstance(value, SummaryData)
        
        # SummaryData object
        data = value
        
        # Initialize cell types
        visits = getVisitsRecursively(instance)
        for visit in visits:
            
            for row in self.rows:
                fieldId = row[0]                
                if self.isTreatmentField(fieldId):
                    # Use treatment object as the source for fields
                    object = Acquisition.aq_parent(visit)
                else:
                    object = visit
                    
                self.updateHostObject(data, visit.getId(), fieldId, object, fieldId)                           
                                                
    def isValid(self, instance):
        """ Summary can only be displayed if we have at least one treatment with one visit """
        
        # Use objectIds() which is faster than listFolderContents()
        for treatmentId in instance.objectIds():
            treatment = instance[treatmentId]            
            objectIds = treatment.objectIds()
            if len(objectIds) >= 1:
                # Found a visit
                return True
            
        return False
            
class SummaryDemoPatient(ATCTFolder):
    """ A person who visits in a hospital """
    
    meta_type = portal_type = archetype_name = 'Patient'
    
    allowed_content_types = ["Treatment"] 

    schema = ATCTFolder.schema.copy()  + Schema((    
                TextField("bloodType",
                  vocabulary=['A','B',"O"],
                  widget=SelectionWidget()
                ),
                ComplexSummaryField(
                    "patientSummary",
                ),                
        ))
    
finalizeATCTSchema(SummaryDemoPatient.schema)       
registerType(SummaryDemoPatient, PROJECTNAME)

class SummaryDemoTreatment(ATCTFolder):
    """ Each patient has N treatments """
    
    meta_type = portal_type = archetype_name = 'Treatment'
    
    allowed_content_types = ["Visit"]     
    
    schema = ATCTFolder.schema.copy()  + Schema((                                                                          
    
            StringField('diseaseName',
                    ),                                                                                                    
        )                          
    )    
    
finalizeATCTSchema(SummaryDemoTreatment.schema)
registerType(SummaryDemoTreatment, PROJECTNAME)
    
class SummaryDemoVisit(ATCTContent):
    """ Each treatment has N visits """
    
    meta_type = portal_type = archetype_name = 'Visit'
    
    schema = ATCTContent.schema.copy()  + Schema((                                                                          
    
            FloatField('bloodPressure',
            ),            
                                                                                              
            IntegerField('cost',
            ),  
            
            StringField('patientAction',
              vocabulary = "getPatientActionVocabulary", 
              widget=SelectionWidget(
                     label="Taken action",
                     ),    
            ),
                                                                   
        ))
    
    def getPatientActionVocabulary(self):
        return DisplayList(    
            (("", "Please choose action",),
             ("home", "Sent to home",),
             ("further", "Further laboratory samples are needed",),             
             ("hospital", "Hospitalized",),         
             ("dead", "Lost him/her",),         
            ))

finalizeATCTSchema(SummaryDemoVisit.schema)    
registerType(SummaryDemoVisit, PROJECTNAME)

    