"""
    
    Summary <input> field handlers
    
    Copyright 2007 Red Innovation http://www.redinnovation.com
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass

class Summarizer:
    """ Manage summary information of one cell 
    
    Mainly functions dealing with HTML form processing
    """
    security = ClassSecurityInfo()        
    
    def __init__(self, allowEmpty=True):
        """
        
        @param allowEmpty Empty summary input values are not written to host objects
        """
        self.allowEmpty = allowEmpty
        
    def isEmptyIgnored(self):
        """ Don't consider empty input fields when updating host objects """
        return self.allowEmpty
     
    security.declarePublic('getTemplate')   
    def getTemplate(self):
        """ Return page template name used for this summarized
        
        Must contain "view" and "edit" macros.
        
        @return e.g. "summary_text"
        
        """
        assert "Abstract method"    
                    
    def read(self, form, field, columnId, rowId):
        """ Read text value from HTML form for certain cell        
        """
        id = self.getInputName(field, columnId, rowId)
        if id in form:
            value = form[id]        
        else: 
            # Form control is disabled - value was not submitted
            value = None
        return value        
                 
    def process(self, value):
        """ Convert HTML form text to object """
        return value
        
    def validate(self, value):
        """ Returns whether a value is good or not 
        
        @return False if validation fails, True if success
        """
        return True
    
    security.declarePublic('getInputName')
    def getInputName(self, field, columnId, rowId, zopify=False):
        """ Return <input> attribute name for HTML code 
        
        @param zopify Add Zope form management magic word 
               (for edit template <input> ids)
        """
        if zopify:
            return "%s_%s_%s" % (field.getName(), columnId, rowId)    
        else:
            return "%s_%s_%s" % (field.getName(), columnId, rowId)    
     
    security.declarePublic('formatOutput')   
    def formatOutput(self, value):    
        """ HTML output of summarized field value 
        
        Can round decimals etc.
        """
        return value
    
    def fetchCellValue(self, data, col, row, object, fieldId, mode):
        """ Helper function for SummaryField.get()
        
        Get one value in cell to view or edit
        
        @param data SummaryData object
        @param col Column id
        @param row Row id
        @param Object Host object (provided by caller functions)
        @param mode One of SummaryField.SUMMARY_XXX constants
        """
        pass
    
    def updateHostObject(self, data, col, row, object, fieldId):
        """ Helper function for SummaryField.set()
        
        Update the field value in the host object
        
        @param data SummaryData object
        @param col Column id
        @param row Row id
        @param Object Host object (provided by caller functions)
        @param fieldId Field name        
        """
        pass        
                    
# Initializes class security
InitializeClass(Summarizer)           

class DummySummarizer(Summarizer):
    """ Filler cell with no action 
    
    Don't touch any objects
    
    Can be overriden to create computed cell values
    """
    def getTemplate(self):
        return "summary_dummy"    
    
    def process(self, value):
        return None

class TextSummarizer(Summarizer):
    """ Handle plain text fields """
    
    def fetchCellValue(self, data, col, row, object, fieldId, mode):
        """ Helper function for SummaryField.get()
        
        Get one value in cell to view or edit
        """
        field = object.schema[fieldId]
        
        # Security check 
        field.checkPermission("r", object)
        value = field.get(object)
        data.setValue(col, row, value)    
        
    def updateHostObject(self, data, col, row, object, fieldId):
        """ Helper function for SummaryField.set()
        
        Update the field value in the host object
        """                
        field = object.schema[fieldId]
        
        # Security check 
        field.checkPermission("w", object)                
        
        # Get plain text value
        read = data.getValue(col, row)                
        
        if read != None and read != "":        
            # Store converted value to the host object
        
            # This assert is not effective unless SummaryField.initSummaryData
            # sets locked status in SummaryField.SUMMARY_PROCESS mode
            assert not data.isLocked(col, row), "Tried to update locked cell, col %s row %s" % (col, row)
        
            field.set(object, self.process(read))               
        else:
            # Value was locked and HTTP submit ignored it
            pass
    
    
    def getTemplate(self):
        return "summary_text"
                           
class SelectionSummarizer(TextSummarizer):
    """ Show selection widget on the summary screen """
    
    security = ClassSecurityInfo()        
    
    security.declarePublic('getFriendlyValue')   
    def getFriendlyValue(self, context, data, column, row):        
        """ Return human-readable form from the vocabulary """        
        value = data.getValue(column, row)
        orignalField = data.getOrignalField(column, row)
        orignalObject = data.getOrignalObject(column, row)
        vocab = orignalField.Vocabulary(orignalObject);
        return orignalObject.displayValue(vocab, value)
    
    def getTemplate(self):
        return "summary_selection"
    
# Initializes class security
InitializeClass(SelectionSummarizer)  
        
class FloatSummarizer(TextSummarizer):
    """ Handle summaries of floating point fields """
    
    
    def validate(self, value):
        """ Returns whether a value is good or not 
        
        @return False if validation fails
        """        
        try:
            float(value)
            return True
        except ValueError:
            return False

    def process(self, value):
        return float(value)

    
class IntegerSummarizer(TextSummarizer):
    """ Handle summaries of integer fields """
    
    def validate(self, value):
        """ Returns whether a value is good or not 
        
        @return False if validation fails
        """
        try:
            int(value)
            return True
        except ValueError:
            return False

    def process(self, value):
        return int(value)
        
    

# Dictionary holding widget type -> summarizer pairs   
widgetRegistry = {
   "StringWidget" : TextSummarizer(),
   "TextWidget" : TextSummarizer(),
   "SelectionWidget" : SelectionSummarizer(),
}

# Dictionary holding field type -> summarizer pairs   
fieldRegistry = {
   "IntegerField" : IntegerSummarizer(),
   "FloatField" : FloatSummarizer(),
}