"""
    
    Summary field core
    
    Copyright 2007 Red Innovation http://www.redinnovation.com
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'javadoc' # Use epydoc to generate API documentation

# Zope imports
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass

from Products.Archetypes.interfaces import IBaseFolder
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Registry import registerField

from Products.CMFCore import permissions

from odict import odict
import summarizers


class SummaryCell:
    """ One cell in summary table
        
    """
    
    def __init__(self):
        
        # Data in some from
        # - Object value when doing get()
        # - Unprocessed HTML text input when doing set()
        self.value = None
        
        # one of summarizers class instances
        self.summarizer = None
        # References to orignal Field instances we are summarizing
        # This information can be exploited in the rendering code        
        self.orignalField = None
        
        # Is edit for this cell allowed
        self.locked = False
        
        # To which object this cell is associated with
        # For example, SelectionWidget needs this to get the available vocabulary
        self.orignalObject = None
        

class SummaryData:
    """  Collect summary info to easy-to-digest tabular form 
        
    One can iterate rows and cols to manipulate data.
    
    All data is in nested dictionaries
        {
            column 1 id : row {
                row 1 id : SummaryCell,
                row 2 id : SummaryCell
            }
        
        }
    
    Assumes that you create first columns, then rows.    
    """
            
    security = ClassSecurityInfo()                    
    
    def __init__(self):
        # contains column id - column friendly label pairs
        self.columnHeaders = odict()
        
        # contains row id - row friendly label pairs
        self.rowHeaders = odict()
        
        # SummaryCell instances 
        self.data = dict()    
        
    
    def assertHasCell(self, columnId, rowId):
        """ Check that data is propeply initialized and print helpful error message """
        if columnId in self.data:
            if rowId in self.data[columnId]:
                return
            raise AttributeError, "Missing row %s for column %s" % (rowId, columnId)
        raise AttributeError, "Missing column %s" % columnId
    
    def addColumn(self, columnId, columnName):
        self.columnHeaders[columnId] = columnName            
        # Initialize structures for new column
        self.data[columnId] = {}
        
    def addRow(self, rowId, rowName):
        self.rowHeaders[rowId] = rowName
        # Initialize cell data
        for columnId in self.columnHeaders.keys():
            self.data[columnId][rowId] = SummaryCell()

    def setValue(self, columnId, rowId, value):
        self.assertHasCell(columnId, rowId)        
        self.data[columnId][rowId].value = value
                
    def setSummarizer(self, columnId, rowId, summarizer):
        self.assertHasCell(columnId, rowId)        
        self.data[columnId][rowId].summarizer = summarizer
        
    def setLocked(self, columnId, rowId, locked):
        self.assertHasCell(columnId, rowId)       
        self.data[columnId][rowId].locked = locked

    def setOrignalField(self, columnId, rowId, field):
        """ Each object can be bind to a field instance """
        self.assertHasCell(columnId, rowId)        
        self.data[columnId][rowId].orignalField = field
        
    def setOrignalObject(self, columnId, rowId, object):
        """ Each object can be bind to a host object instance """        
        self.assertHasCell(columnId, rowId)        
        self.data[columnId][rowId].orignalObject = object        
        
    security.declarePublic('getValue')
    def getValue(self, columnId, rowId):
        self.assertHasCell(columnId, rowId)        
        return self.data[columnId][rowId].value
    
    security.declarePublic('getSummarizer')
    def getSummarizer(self, columnId, rowId):
        self.assertHasCell(columnId, rowId)        
        summarizer = self.data[columnId][rowId].summarizer
        if summarizer == None:
            raise AttributeError, "Missing summarizer for col:%s, row:%s" % (columnId, rowId)
        return summarizer
    
    security.declarePublic('isLocked')
    def isLocked(self, columnId, rowId):
        self.assertHasCell(columnId, rowId)        
        return self.data[columnId][rowId].locked
        
    security.declarePublic('getOrignalField')
    def getOrignalField(self, columnId, rowId):
        self.assertHasCell(columnId, rowId)        
        return self.data[columnId][rowId].orignalField
    
    security.declarePublic('getOrignalObject')
    def getOrignalObject(self, columnId, rowId):
        self.assertHasCell(columnId, rowId)        
        return self.data[columnId][rowId].orignalObject    
    
    security.declarePublic('getColumnIds')
    def getColumnIds(self):            
        return self.columnHeaders.keys()
    
    security.declarePublic('getColumnLabels')
    def getColumnLabels(self):            
        return self.columnHeaders.values()    
    
    security.declarePublic('getRowIds')
    def getRowIds(self):    
        return self.rowHeaders.keys() 
    
    security.declarePublic('getRowLabel')
    def getRowLabel(self, rowId):    
        return self.rowHeaders[rowId]
                            
# Initializes class security
InitializeClass(SummaryData)                           

class SummaryWidget(TypesWidget):
    """ Tabular view for nested data
    
    Each HTML <input> id is in form::
            
        containerSummaryFieldId_objectId_objectFieldId
        
    
    """

    _properties = TypesWidget._properties.copy()    
    _properties.update({
        'macro' : "summary_widget",
        
        # CSS for styling SummaryWidget
        'helper_css' : ('summary.css',),
        
        # Label for 1st column header
        'masterLabel' : "Item",
 
        
        })
    
    security = ClassSecurityInfo()    
    
    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """ Read incoming HTTP POST
        
        Read incoming HTTP request for form submit. 
        
        Convert all <input> data to SummaryData table. 
        Data is still in plain-text form.
        
        @return SummaryData instance to SummaryField.set to process        
        """
        
        if not field.isValid(instance):
            # Folder object has been created,
            # initial form subit
            # there aren't any child content yet
            # and initializeSummary needs some
            return (None, {})
        
        data = field.initializeSummaryData(instance, mode=field.SUMMARY_PROCESS)
                                        
        for columnId in data.getColumnIds():
            for rowId in data.getRowIds():
                
                summarizer = data.getSummarizer(columnId, rowId)                
                
                read = summarizer.read(form, field, columnId, rowId)
                data.setValue(columnId, rowId, read)
        
        return (data, {})    

    security.declarePublic('getTemplate')
    def getTemplate(self, fieldType):
        return summarizers.registry[fieldType].getTemplate()
    
    security.declarePublic('getData')
    def getData(self, instance, field, mode):
        """ Return data for widget template to render """
        # SummaryField.get() should handle security as per
        # field basis        
        return field.get(instance, mode=mode)
        
    security.declarePublic('getMasterLabel')
    def getMasterLabel(self, instance, field):
        """ Return column header for the first column """        
        return self.masterLabel
        
registerWidget(SummaryWidget,
               title='SummaryWidget',
               description='Tabular view for summarized data',
               used_for="Products.summary.core.SummaryField"
               )
      
        
class SummaryField(ObjectField):
    """ Summary field allows quickly edit nested content fields
    
    A table from nested content is generated for editing
    

    """
    
    # Pseudo constants used in initSummaryData
    
    # Summary is generated for AT object view template
    SUMMARY_VIEW = "view"
    
    # Summary is generated for AT object edit template
    SUMMARY_EDIT = "edit"
    
    # Summary is generated for HTTP POST of form data
    SUMMARY_PROCESS = "process"
    
        
    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'summary',
        'mode' : 'rw',
        
        'widget' : SummaryWidget,
        
        'validators' : [],
        
        # Function to perform locked chell checks
        'locker' : None,
        
        # List of NestedField instances
        'nestedFields' : [],
        
        # Dictionary of column id -> Summarizer instance pairs
        # Override summarizer by column type
        'columnSummarizers' : {},
        
        # Dictionary of row id -> Summarizer instance pairs
        # Override summarizer by row type
        'rowSummarizers' : {},               
        })

    security = ClassSecurityInfo()    

    def __init__(self, name=None, **kwargs):
        """ Create DataGridField instance
        """

        # call super constructor
        ObjectField.__init__(self, name, **kwargs)
                
    security.declarePrivate('initializeSummaryData')
    def initializeSummaryData(self, instance, mode):
        """ Prepares SummaryData instance
        
        Builds table structure        
            - Column labels
            - Row labels
            - Cell data types        
        
        @param instance Object whose summaryfield we are dealing with
        
        @param mode One of SummaryField.SUMMARY_XXX constants
            defining for what purpose we are generating the table structure
        """
        
        data = SummaryData()
        
        objects = self.getSummarizedObjects(instance)

        # Initialize row labels
        sample = objects[0]
        for fieldId in self.nestedFields:
            field = sample.schema[fieldId]            
            data.addColumn(fieldId, field.getName())

        # Initialize column labels        
        for object in objects:
            data.addRow(object.getId(), object.Title())
        
        # Initialize cell types
        objects = self.getSummarizedObjects(instance)
        for object in objects:
            for fieldId in self.nestedFields:
                self.initCell(data, fieldId, object.getId(), object, fieldId)
                        
        return data
    
    def initCell(self, data, col, row, object, fieldId):
        """ Initialize Summary data cell using given host field
        
        Each table cell is bind to field. Field type is
        used to determine what kind of cell formatter is 
        used for the data.        
        """        
        field = object.schema[fieldId]        
        summarizer = self.mapSummarizer(col, row, field)
        data.setOrignalField(col, row, field)
        data.setSummarizer(col, row, summarizer)
        data.setOrignalObject(col, row, object)
        
    def fetchCellValue(self, data, col, row, object, fieldId, mode):      
        """ Helper function for get()
        
        Get value for a cell from the host object
        """
        summarizer = data.getSummarizer(col, row)
        return summarizer.fetchCellValue(data, col, row, object, fieldId, mode)
    
    def updateHostObject(self, data, col, row, object, fieldId):
        """ Helper function for set()
        
        Update the field value in the host object
        """
        summarizer = data.getSummarizer(col, row)
        return summarizer.updateHostObject(data, col, row, object, fieldId)        
    
    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        """ Return SummaryData object
        
        For every object create a column.
        For every child item create a row.
        
        Children can override this for custom behavior.
        
        @param instance Context object which has summary field
        
        @param mode (optional) one of SummaryField.SUMMARY_XXX constants
        
        @return Summary data in SummaryData instance format
        """
        
        if not IBaseFolder.isImplementedBy(instance):
            raise ValueError, "Summary field can be placed on folderish objects only"
         
        # Templates use special mode to generate edit table structure
        if "mode" in kwargs:
            mode = kwargs["mode"]
        else:
            mode=SummaryField.SUMMARY_VIEW
            
        data = self.initializeSummaryData(instance, mode)        
        
        # For every object create a column
        # For every child item create a row        
        for object in self.getSummarizedObjects(instance):
            values = odict()
                        
            for fieldId in self.nestedFields:                
                self.fetchCellValue(data, fieldId, object.getId(), object, fieldId, mode)
                                            
        return data

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        """ Sets new data 
        
        By this point, every SummaryData cell contains textual 
        validated, value, of form input. SummaryField.set
        converts this data to objects and passes data forward
        to storage containers.
        
        @param value SummaryData with cols and rows, formed by widget
        
        """                          
        
        if value == None:
            # No summarized objects yet
            return         
        
        
        assert isinstance(value, SummaryData)
        
        # SummaryData object
        data = value

        fields = self.nestedFields
          
        for fieldId in data.getColumnIds():
            for objectId in data.getRowIds():
                object = instance[objectId]            
                self.updateHostObject(data, fieldId, objectId, object, fieldId)
                
    security.declarePublic('validate')
    def validate(self, value, instance, errors=None, **kwargs):
        """
        Validate passed-in value using all field validators.
        Return None if all validations pass; otherwise, return failed
        result returned by validator
        
        @param errors Dictionary of field id : list of error strings pairs
        """
           
        if errors is None:
            errors = {}
            
        if value == None:
            # No summarized objects yet
            return
        
        assert isinstance(value, SummaryData)        
        data = value 
        
        #
        # Iterate through all cells
        # Get summarizer objects for them
        # and run form data against summarizer validator,
        # Bad summary cell ids are marked to errors dictionary
        #
        for columnId in data.getColumnIds():
            for rowId in data.getRowIds():
                                
                summarizer = data.getSummarizer(columnId, rowId)     
                value = data.getValue(columnId, rowId)
                fieldId = summarizer.getInputName(self, columnId, rowId)                
                
                if value == "" and summarizer.isEmptyIgnored():
                    continue
                
                if not summarizer.validate(value):
                    errors[fieldId] = "Validation failed"
                            
        if len(errors) > 0:
            # Per summary field error message
            return "Summary input data had errors"
        
        return None
                
    
    def getSummarizedObjects(self, instance):
        """ Return list of objects which appear in summary 
        
        Can be overridden, by default returns all objects
        """
        return instance.listFolderContents()
        
    def isValid(self, instance):
        """ Check if the summary can be formed
        
        We need at least one child object to get field label data
        """
        
        # TODO: optimize for large loads
        objects = self.getSummarizedObjects(instance)        
        return len(objects) > 0
    
    security.declarePublic('getSummarizer')
    def mapSummarizer(self, columnId, rowId, field):
        """ Get data handler for certain column/row/type 
        
        Look up order
        
        1. Per-field column definitions
        2. Per-field row definitions
        3. Global registry by host field class name
        4. Global registry by host widget class name
        """
        if columnId in self.columnSummarizers:
            return self.columnSummarizers[columnId]        
        elif rowId in self.rowSummarizers:
            return self.rowSummarizers[columnId]
        elif field.__class__.__name__ in summarizers.fieldRegistry:
            return summarizers.fieldRegistry[field.__class__.__name__]
        elif field.widget.__class__.__name__ in summarizers.widgetRegistry:
            return summarizers.widgetRegistry[field.widget.__class__.__name__]
        
        raise ValueError, "Unknown summarizer %s %s %s" % (columnId, rowId, field)
        
                                                                                            
registerField(SummaryField,
              title='SummaryField',
              description='Quickly view and edit summarized content fields')

class LockFilledSummaryField(SummaryField):
    """ Summary field which disallows editing already filled values """

    def isLocked(self, object, columId, rowId, field, value):
        """ Disable editing the field
        
        Children should override
        """ 
        return value != ""
                                                                            
registerField(LockFilledSummaryField,
              title='LockFilledSummaryField',
              description='Summary field allows quickly edit unfilled nested content fields')    
        
# Exported Python objects
__all__ = ('SummaryData', 'SummaryField', 'LockFilledSummaryField', 'SummaryWidget')
